@@ -40,6 +40,7 @@
 
 #include "cpl_conv.h"
 #include "cpl_error.h"
+#include "cpl_mem_cache.h"
 #include "cpl_minixml.h"
 #include "cpl_string.h"
 #include "cpl_vsi.h"
@@ -57,7 +58,7 @@
 
 // #define DEBUG_VERBOSE_EXTRACT_DEM
 
-CPL_CVSID("$Id: gdal_rpc.cpp 9a6c1bfc5c760e68e084b2ddfcd439b96edb4caf 2020-06-15 02:55:34 -0700 Yann-Sebastien Tremblay-Johnston $")
+CPL_CVSID("$Id$")
 
 CPL_C_START
 CPLXMLNode *GDALSerializeRPCTransformer( void *pTransformArg );
@@ -73,12 +74,34 @@ constexpr double DEFAULT_PIX_ERR_THRESHOLD = 0.1;
 /*      Turn an RPCInfo structure back into its metadata format.        */
 /************************************************************************/
 
-char ** RPCInfoToMD( GDALRPCInfo *psRPCInfo )
+char ** RPCInfoV1ToMD( GDALRPCInfoV1 *psRPCInfo )
+
+{
+    GDALRPCInfoV2 sRPCInfo;
+    memcpy(&sRPCInfo, psRPCInfo, sizeof(GDALRPCInfoV1));
+    sRPCInfo.dfERR_BIAS = std::numeric_limits<double>::quiet_NaN();
+    sRPCInfo.dfERR_RAND = std::numeric_limits<double>::quiet_NaN();
+    return RPCInfoV2ToMD(&sRPCInfo);
+}
+
+char ** RPCInfoV2ToMD( GDALRPCInfoV2 *psRPCInfo )
 
 {
     char **papszMD = nullptr;
     CPLString osField, osMultiField;
 
+    if( !std::isnan(psRPCInfo->dfERR_BIAS) )
+    {
+        osField.Printf( "%.15g", psRPCInfo->dfERR_BIAS );
+        papszMD = CSLSetNameValue( papszMD, RPC_ERR_BIAS, osField );
+    }
+
+    if( !std::isnan(psRPCInfo->dfERR_RAND) )
+    {
+        osField.Printf( "%.15g", psRPCInfo->dfERR_RAND );
+        papszMD = CSLSetNameValue( papszMD, RPC_ERR_RAND, osField );
+    }
+
     osField.Printf( "%.15g", psRPCInfo->dfLINE_OFF );
     papszMD = CSLSetNameValue( papszMD, RPC_LINE_OFF, osField );
 
@@ -216,7 +239,7 @@ typedef struct {
 
     GDALTransformerInfo sTI;
 
-    GDALRPCInfo sRPC;
+    GDALRPCInfoV2 sRPC;
 
     double      adfPLToLatLongGeoTransform[6];
     double      dfRefZ;
@@ -239,16 +262,8 @@ typedef struct {
     int         bApplyDEMVDatumShift;
 
     GDALDataset *poDS;
-    double     *padfDEMBuffer;
-    int         nDEMExtractions;
-    int         nBufferMaxRadius;
-    int         nHitsInBuffer;
-    int         nBufferX;
-    int         nBufferY;
-    int         nBufferWidth;
-    int         nBufferHeight;
-    int         nLastQueriedX;
-    int         nLastQueriedY;
+    // the key is (nYBlock << 32) | nXBlock)
+    lru11::Cache<uint64_t, std::shared_ptr<std::vector<double>>>* poCacheDEM;
 
     OGRCoordinateTransformation *poCT;
 
@@ -482,8 +497,8 @@ void* GDALCreateSimilarRPCTransformer( void *hTransformArg,
     GDALRPCTransformInfo *psInfo =
         static_cast<GDALRPCTransformInfo *>(hTransformArg);
 
-    GDALRPCInfo sRPC;
-    memcpy(&sRPC, &(psInfo->sRPC), sizeof(GDALRPCInfo));
+    GDALRPCInfoV2 sRPC;
+    memcpy(&sRPC, &(psInfo->sRPC), sizeof(GDALRPCInfoV2));
 
     if( dfRatioX != 1.0 || dfRatioY != 1.0 )
     {
@@ -517,7 +532,7 @@ void* GDALCreateSimilarRPCTransformer( void *hTransformArg,
                                    CPLSPrintf("%d", psInfo->nMaxIterations));
 
     GDALRPCTransformInfo* psNewInfo =
-        static_cast<GDALRPCTransformInfo*>(GDALCreateRPCTransformer(
+        static_cast<GDALRPCTransformInfo*>(GDALCreateRPCTransformerV2(
             &sRPC, psInfo->bReversed, psInfo->dfPixErrThreshold, papszOptions));
     CSLDestroy(papszOptions);
 
@@ -626,6 +641,20 @@ retry:
 /*                      GDALCreateRPCTransformer()                      */
 /************************************************************************/
 
+
+void *GDALCreateRPCTransformerV1( GDALRPCInfoV1 *psRPCInfo, int bReversed,
+                                double dfPixErrThreshold,
+                                char **papszOptions )
+
+{
+    GDALRPCInfoV2 sRPCInfo;
+    memcpy(&sRPCInfo, psRPCInfo, sizeof(GDALRPCInfoV1));
+    sRPCInfo.dfERR_BIAS = std::numeric_limits<double>::quiet_NaN();
+    sRPCInfo.dfERR_RAND = std::numeric_limits<double>::quiet_NaN();
+    return GDALCreateRPCTransformerV2(&sRPCInfo, bReversed, dfPixErrThreshold,
+                                      papszOptions);
+}
+
 /**
  * Create an RPC based transformer.
  *
@@ -778,7 +807,7 @@ retry:
  * @return transformer callback data (deallocate with GDALDestroyTransformer()).
  */
 
-void *GDALCreateRPCTransformer( GDALRPCInfo *psRPCInfo, int bReversed,
+void *GDALCreateRPCTransformerV2( const GDALRPCInfoV2 *psRPCInfo, int bReversed,
                                 double dfPixErrThreshold,
                                 char **papszOptions )
 
@@ -789,7 +818,7 @@ void *GDALCreateRPCTransformer( GDALRPCInfo *psRPCInfo, int bReversed,
     GDALRPCTransformInfo *psTransform = static_cast<GDALRPCTransformInfo *>(
         CPLCalloc(sizeof(GDALRPCTransformInfo), 1));
 
-    memcpy( &(psTransform->sRPC), psRPCInfo, sizeof(GDALRPCInfo) );
+    memcpy( &(psTransform->sRPC), psRPCInfo, sizeof(GDALRPCInfoV2) );
     psTransform->bReversed = bReversed;
     const char* pszPixErrThreshold =
         CSLFetchNameValue( papszOptions, "RPC_PIXEL_ERROR_THRESHOLD" );
@@ -941,7 +970,7 @@ void *GDALCreateRPCTransformer( GDALRPCInfo *psRPCInfo, int bReversed,
             if( OGRHasPreparedGeometrySupport() )
             {
                 psTransform->poRPCFootprintPreparedGeom =
-                    OGRCreatePreparedGeometry(psTransform->poRPCFootprintGeom);
+                    OGRCreatePreparedGeometry(OGRGeometry::ToHandle(psTransform->poRPCFootprintGeom));
             }
             else
             {
@@ -1080,7 +1109,7 @@ void GDALDestroyRPCTransformer( void *pTransformAlg )
 
     if( psTransform->poDS )
         GDALClose(psTransform->poDS);
-    CPLFree(psTransform->padfDEMBuffer);
+    delete psTransform->poCacheDEM;
     if( psTransform->poCT )
         OCTDestroyCoordinateTransformation(
             reinterpret_cast<OGRCoordinateTransformationH>(psTransform->poCT));
@@ -1376,113 +1405,91 @@ static bool GDALRPCExtractDEMWindow( GDALRPCTransformInfo *psTransform,
                                      int nX, int nY, int nWidth, int nHeight,
                                      double* padfOut )
 {
-    psTransform->nDEMExtractions++;
-    if( psTransform->padfDEMBuffer == nullptr )
+    constexpr int BLOCK_SIZE = 64;
+
+    // Request the DEM by blocks of BLOCK_SIZE * BLOCK_SIZE and put them
+    // in poCacheDEM
+    if( psTransform->poCacheDEM == nullptr )
+        psTransform->poCacheDEM = new lru11::Cache<uint64_t, std::shared_ptr<std::vector<double>>>();
+
+    const int nXIters = (nX + nWidth - 1) / BLOCK_SIZE - nX / BLOCK_SIZE + 1;
+    const int nYIters = (nY + nHeight - 1) / BLOCK_SIZE - nY / BLOCK_SIZE + 1;
+    const int nRasterXSize = psTransform->poDS->GetRasterXSize();
+    const int nRasterYSize = psTransform->poDS->GetRasterYSize();
+    for(int iY = 0; iY < nYIters; iY++)
     {
-        // Should only happen in case of failed memory allocation.
-        return psTransform->poDS->GetRasterBand(1)->
-                                  RasterIO(GF_Read, nX, nY, nWidth, nHeight,
-                                           padfOut, nWidth, nHeight,
-                                           GDT_Float64, 0, 0,
-                                           nullptr) == CE_None;
+        const int nBlockY = nY / BLOCK_SIZE + iY;
+        const int nReqYSize = std::min( nRasterYSize - nBlockY * BLOCK_SIZE, BLOCK_SIZE );
+        const int nFirstLineInCachedBlock = (iY == 0) ? nY % BLOCK_SIZE : 0;
+        const int nFirstLineInOutput = (iY == 0) ? 0 : BLOCK_SIZE - (nY % BLOCK_SIZE) + (iY - 1) * BLOCK_SIZE;
+        const int nLinesToCopy =
+            (nYIters == 1) ? nHeight :
+            (iY == 0) ? BLOCK_SIZE - (nY % BLOCK_SIZE) :
+            (iY == nYIters - 1) ? 1 + (nY + nHeight - 1) % BLOCK_SIZE :
+            BLOCK_SIZE;
+        for(int iX = 0; iX < nXIters; iX++)
+        {
+            const int nBlockX = nX / BLOCK_SIZE + iX;
+            const int nReqXSize = std::min( nRasterXSize - nBlockX * BLOCK_SIZE, BLOCK_SIZE );
+            const uint64_t nKey = (static_cast<uint64_t>(nBlockY) << 32) | nBlockX;
+            const int nFirstColInCachedBlock = (iX == 0) ? nX % BLOCK_SIZE : 0;
+            const int nFirstColInOutput = (iX == 0) ? 0 : BLOCK_SIZE - (nX % BLOCK_SIZE) + (iX - 1) * BLOCK_SIZE;
+            const int nColsToCopy =
+                (nXIters == 1) ? nWidth :
+                (iX == 0) ? BLOCK_SIZE - (nX % BLOCK_SIZE) :
+                (iX == nXIters - 1) ? 1 + (nX + nWidth - 1) % BLOCK_SIZE :
+                BLOCK_SIZE;
+
+#if 0
+            CPLDebug("RPC", "nY=%d nX=%d nBlockY=%d nBlockX=%d "
+                     "nFirstLineInCachedBlock=%d nFirstLineInOutput=%d nLinesToCopy=%d "
+                     "nFirstColInCachedBlock=%d nFirstColInOutput=%d nColsToCopy=%d",
+                     nY, nX, nBlockY, nBlockX, nFirstLineInCachedBlock, nFirstLineInOutput, nLinesToCopy,
+                     nFirstColInCachedBlock, nFirstColInOutput, nColsToCopy);
+#endif
+
+            std::shared_ptr<std::vector<double>> poValue;
+            if( !psTransform->poCacheDEM->tryGet(nKey, poValue) )
+            {
+                poValue = std::make_shared<std::vector<double>>(nReqXSize * nReqYSize);
+                CPLErr eErr = psTransform->poDS->GetRasterBand(1)->RasterIO(GF_Read,
+                                nBlockX * BLOCK_SIZE, nBlockY * BLOCK_SIZE,
+                                nReqXSize, nReqYSize,
+                                poValue->data(),
+                                nReqXSize, nReqYSize,
+                                GDT_Float64,
+                                0, 0, nullptr);
+                if( eErr != CE_None )
+                {
+                    return false;
+                }
+                psTransform->poCacheDEM->insert(nKey, poValue);
+            }
+
+            // Compose the cached block to the final buffer
+            for( int j=0; j<nLinesToCopy; j++)
+            {
+                memcpy( padfOut + (nFirstLineInOutput + j) * nWidth + nFirstColInOutput,
+                        poValue->data() + (nFirstLineInCachedBlock + j) * nReqXSize + nFirstColInCachedBlock,
+                        nColsToCopy * sizeof(double) );
+            }
+        }
     }
 
-    // Instead of reading just nWidth * nHeight pixels (with those being <= 4),
-    // target a larger buffer since small extractions can be costly, particular
-    // with VRT.
-    if( !(nX >= psTransform->nBufferX &&
-          nX + nWidth <= psTransform->nBufferX + psTransform->nBufferWidth &&
-          nY >= psTransform->nBufferY &&
-          nY + nHeight <= psTransform->nBufferY + psTransform->nBufferHeight ) )
+#if 0
+    CPLDebug("RPC_DEM", "DEM for %d,%d,%d,%d", nX, nY, nWidth, nHeight);
+    for(int j = 0; j < nHeight; j++)
     {
-#ifdef DEBUG_VERBOSE_EXTRACT_DEM
-        CPLDebug("RPC", "Current request: %d,%d-%dx%d",
-                  nX, nY, nWidth, nHeight);
-        CPLDebug("RPC", "Hits in last DEM buffer: %d (%d pixels)",
-                 psTransform->nHitsInBuffer,
-                 psTransform->nBufferWidth * psTransform->nBufferHeight);
-        CPLDebug("RPC", "Last DEM buffer: %d,%d-%dx%d",
-                 psTransform->nBufferX,
-                 psTransform->nBufferY,
-                 psTransform->nBufferWidth,
-                 psTransform->nBufferHeight);
-#endif
-        const int nRasterXSize = psTransform->poDS->GetRasterXSize();
-        const int nRasterYSize = psTransform->poDS->GetRasterYSize();
-        // If we have only queried a few points, no need to extract on a large
-        // window We will progressively extend the window up to its maximum size
-        // if we extract a significant number of points.
-        int nRadius = psTransform->nBufferMaxRadius;
-        if( psTransform->nDEMExtractions <
-            psTransform->nBufferMaxRadius * psTransform->nBufferMaxRadius )
+        std::string osLine;
+        for(int i = 0; i < nWidth; ++i )
         {
-            nRadius = static_cast<int> (
-                  sqrt(static_cast<double>(psTransform->nDEMExtractions)) );
-            CPLAssert( nRadius <= psTransform->nBufferMaxRadius );
+            if( !osLine.empty() )
+                osLine += ", ";
+            osLine += std::to_string(padfOut[j * nWidth + i]);
         }
-        // Check if there's some overlap between consecutive requests to decide
-        // if we must have a buffer around the interest window.
-        const int nDiffX = nX - psTransform->nLastQueriedX;
-        const int nDiffY = nY - psTransform->nLastQueriedY;
-        if( psTransform->nLastQueriedX >= 0 &&
-            (nDiffX > nRadius || -nDiffX > nRadius ||
-             nDiffY > nRadius || -nDiffY > nRadius) )
-        {
-            nRadius = 0;
-        }
-        psTransform->nBufferX = nX - nRadius;
-        if( psTransform->nBufferX < 0 )
-            psTransform->nBufferX = 0;
-        psTransform->nBufferY = nY - nRadius;
-        if( psTransform->nBufferY < 0 )
-            psTransform->nBufferY = 0;
-        psTransform->nBufferWidth = nWidth + 2 * nRadius;
-        if( psTransform->nBufferX + psTransform->nBufferWidth > nRasterXSize )
-            psTransform->nBufferWidth = nRasterXSize - psTransform->nBufferX;
-        psTransform->nBufferHeight = nHeight + 2 * nRadius;
-        if( psTransform->nBufferY + psTransform->nBufferHeight > nRasterYSize )
-            psTransform->nBufferHeight = nRasterYSize - psTransform->nBufferY;
-#ifdef DEBUG_VERBOSE_EXTRACT_DEM
-        CPLDebug("RPC", "New DEM buffer: %d,%d-%dx%d",
-                 psTransform->nBufferX,
-                 psTransform->nBufferY,
-                 psTransform->nBufferWidth,
-                 psTransform->nBufferHeight);
-#endif
-        CPLErr eErr = psTransform->poDS->GetRasterBand(1)->RasterIO(GF_Read,
-                        psTransform->nBufferX, psTransform->nBufferY,
-                        psTransform->nBufferWidth, psTransform->nBufferHeight,
-                        psTransform->padfDEMBuffer,
-                        psTransform->nBufferWidth, psTransform->nBufferHeight,
-                        GDT_Float64, 0, 0, nullptr);
-        if( eErr != CE_None )
-        {
-            psTransform->nBufferX = -1;
-            psTransform->nBufferY = -1;
-            psTransform->nBufferWidth = -1;
-            psTransform->nBufferHeight = -1;
-            return false;
-        }
-#ifdef DEBUG_VERBOSE_EXTRACT_DEM
-         psTransform->nHitsInBuffer = 1;
-#endif
+        CPLDebug("RPC_DEM", "%s", osLine.c_str());
     }
-    else
-    {
-#ifdef DEBUG_VERBOSE
-        psTransform->nHitsInBuffer++;
 #endif
-    }
-    psTransform->nLastQueriedX = nX;
-    psTransform->nLastQueriedY = nY;
-
-    for( int i=0; i<nHeight; i++)
-    {
-        memcpy( padfOut + i * nWidth,
-                psTransform->padfDEMBuffer + (nY - psTransform->nBufferY + i) *
-                    psTransform->nBufferWidth + nX - psTransform->nBufferX,
-                nWidth * sizeof(double) );
-    }
 
     return true;
 }
@@ -1650,7 +1657,7 @@ static bool RPCIsValidLongLat( const GDALRPCTransformInfo *psTransform,
     OGRPoint p(dfLong, dfLat);
     return CPL_TO_BOOL(
         OGRPreparedGeometryContains(psTransform->poRPCFootprintPreparedGeom,
-                                       &p));
+                                    OGRGeometry::ToHandle(&p)));
 }
 
 /************************************************************************/
@@ -1911,21 +1918,6 @@ static bool GDALRPCOpenDEM( GDALRPCTransformInfo* psTransform )
     if( psTransform->poDS != nullptr &&
         psTransform->poDS->GetRasterCount() >= 1 )
     {
-        psTransform->nBufferMaxRadius =
-            atoi(CPLGetConfigOption("GDAL_RPC_DEM_BUFFER_MAX_RADIUS", "2"));
-        psTransform->nHitsInBuffer = 0;
-        const int nMaxWindowSize = 4;
-        psTransform->padfDEMBuffer = static_cast<double*>(VSIMalloc(
-            (nMaxWindowSize + 2 * psTransform->nBufferMaxRadius) *
-            (nMaxWindowSize + 2 * psTransform->nBufferMaxRadius) *
-            sizeof(double) ));
-        psTransform->nBufferX = -1;
-        psTransform->nBufferY = -1;
-        psTransform->nBufferWidth = -1;
-        psTransform->nBufferHeight = -1;
-        psTransform->nLastQueriedX = -1;
-        psTransform->nLastQueriedY = -1;
-
         OGRSpatialReference oDEMSRS;
         if ( psTransform->pszDEMSRS != nullptr )
         {
@@ -2316,7 +2308,7 @@ CPLXMLNode *GDALSerializeRPCTransformer( void *pTransformArg )
 /* -------------------------------------------------------------------- */
 /*      RPC metadata.                                                   */
 /* -------------------------------------------------------------------- */
-    char **papszMD = RPCInfoToMD( &(psInfo->sRPC) );
+    char **papszMD = RPCInfoV2ToMD( &(psInfo->sRPC) );
     CPLXMLNode *psMD= CPLCreateXMLNode( psTree, CXT_Element,
                                         "Metadata" );
 
@@ -2376,8 +2368,8 @@ void *GDALDeserializeRPCTransformer( CPLXMLNode *psTree )
                              psMDI->psChild->psNext->pszValue );
     }
 
-    GDALRPCInfo sRPC;
-    if( !GDALExtractRPCInfo( papszMD, &sRPC ) )
+    GDALRPCInfoV2 sRPC;
+    if( !GDALExtractRPCInfoV2( papszMD, &sRPC ) )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "Failed to reconstitute RPC transformer." );
@@ -2432,7 +2424,7 @@ void *GDALDeserializeRPCTransformer( CPLXMLNode *psTree )
 /*      Generate transformation.                                        */
 /* -------------------------------------------------------------------- */
     void *pResult =
-        GDALCreateRPCTransformer( &sRPC, bReversed, dfPixErrThreshold,
+        GDALCreateRPCTransformerV2( &sRPC, bReversed, dfPixErrThreshold,
                                   papszOptions );
 
     CSLDestroy( papszOptions );
