@@ -34,7 +34,7 @@
 #include "rawdataset.h"
 #include "ogr_srs_api.h"
 
-CPL_CVSID("$Id: sar_ceosdataset.cpp 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 static GInt16 CastToGInt16(float val)
 {
@@ -1715,6 +1715,8 @@ void SAR_CEOSDataset::ScanForGCPs()
     /* If general GCP's were not found, look for Map Projection (e.g. JERS) */
     if( nGCPCount == 0 )
     {
+        CPLFree(pasGCPList);
+        pasGCPList = nullptr;
         ScanForMapProjection();
         return;
     }
