@@ -26,6 +26,8 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
+#include "cpl_port.h"
+
 #include "gdal_priv.h"
 #include "gtiff.h"
 #include "gt_overview.h"
@@ -71,8 +73,25 @@ static bool HasZSTDCompression()
 static CPLString GetTmpFilename(const char* pszFilename,
                                 const char* pszExt)
 {
+    const bool bSupportsRandomWrite =
+        !(STARTS_WITH(pszFilename, "/vsitar/") ||
+          STARTS_WITH(pszFilename, "/vsizip/") ||
+          STARTS_WITH(pszFilename, "/vsigzip/") ||
+          STARTS_WITH(pszFilename, "/vsis3") ||
+          STARTS_WITH(pszFilename, "/vsigs") ||
+          STARTS_WITH(pszFilename, "/vsiaz") ||
+          STARTS_WITH(pszFilename, "/vsiadls") ||
+          STARTS_WITH(pszFilename, "/vsioss"));
     CPLString osTmpFilename;
-    osTmpFilename.Printf("%s.%s", pszFilename, pszExt);
+    if( !bSupportsRandomWrite ||
+        CPLGetConfigOption( "CPL_TMPDIR", nullptr ) != nullptr )
+    {
+        osTmpFilename = CPLGenerateTempFilename(CPLGetBasename(pszFilename));
+    }
+    else
+        osTmpFilename = pszFilename;
+    osTmpFilename += '.';
+    osTmpFilename += pszExt;
     VSIUnlink(osTmpFilename);
     return osTmpFilename;
 }
@@ -313,48 +332,63 @@ bool COGGetWarpingCharacteristics(GDALDataset* poSrcDS,
         const auto& tmList = poTM->tileMatrixList();
         const int nBlockSize = atoi(CSLFetchNameValueDef(
             papszOptions, "BLOCKSIZE", CPLSPrintf("%d", tmList[0].mTileWidth)));
+        dfRes = 0.0;
+
+        const char* pszZoomLevel = CSLFetchNameValue(papszOptions, "ZOOM_LEVEL");
+        if( pszZoomLevel )
+        {
+            nZoomLevel = atoi(pszZoomLevel);
+            if( nZoomLevel < 0 || nZoomLevel >= static_cast<int>(tmList.size()) )
+            {
+                CPLError(CE_Failure, CPLE_AppDefined,
+                         "Invalid zoom level: should be in [0,%d]",
+                         static_cast<int>(tmList.size())-1);
+                return false;
+            }
+        }
+        else
+        {
+            double dfComputedRes = adfGeoTransform[1];
+            double dfPrevRes = 0.0;
+            for( ; nZoomLevel < static_cast<int>(tmList.size()); nZoomLevel++ )
+            {
+                dfRes = tmList[nZoomLevel].mResX * tmList[0].mTileWidth / nBlockSize;
+                if( dfComputedRes > dfRes || fabs( dfComputedRes - dfRes ) / dfRes <= 1e-8 )
+                    break;
+                dfPrevRes = dfRes;
+            }
+            if( nZoomLevel == static_cast<int>(tmList.size()) )
+            {
+                CPLError(CE_Failure, CPLE_AppDefined,
+                        "Could not find an appropriate zoom level");
+                return false;
+            }
+
+            if( nZoomLevel > 0 && fabs( dfComputedRes - dfRes ) / dfRes > 1e-8 )
+            {
+                const char* pszZoomLevelStrategy = CSLFetchNameValueDef(papszOptions,
+                                                                        "ZOOM_LEVEL_STRATEGY",
+                                                                        "AUTO");
+                if( EQUAL(pszZoomLevelStrategy, "LOWER") )
+                {
+                    nZoomLevel --;
+                }
+                else if( EQUAL(pszZoomLevelStrategy, "UPPER") )
+                {
+                    /* do nothing */
+                }
+                else
+                {
+                    if( dfPrevRes / dfComputedRes < dfComputedRes / dfRes )
+                        nZoomLevel --;
+                }
+            }
+        }
+        CPLDebug("COG", "Using ZOOM_LEVEL %d", nZoomLevel);
+        dfRes = tmList[nZoomLevel].mResX * tmList[0].mTileWidth / nBlockSize;
+
         const double dfOriX = bInvertAxis ? tmList[0].mTopLeftY : tmList[0].mTopLeftX;
         const double dfOriY = bInvertAxis ? tmList[0].mTopLeftX : tmList[0].mTopLeftY;
-        double dfComputedRes = adfGeoTransform[1];
-        double dfPrevRes = 0.0;
-        dfRes = 0.0;
-        for( ; nZoomLevel < static_cast<int>(tmList.size()); nZoomLevel++ )
-        {
-            dfRes = tmList[nZoomLevel].mResX * tmList[0].mTileWidth / nBlockSize;
-            if( dfComputedRes > dfRes || fabs( dfComputedRes - dfRes ) / dfRes <= 1e-8 )
-                break;
-            dfPrevRes = dfRes;
-        }
-        if( nZoomLevel == static_cast<int>(tmList.size()) )
-        {
-            CPLError(CE_Failure, CPLE_AppDefined,
-                    "Could not find an appropriate zoom level");
-            return false;
-        }
-
-        if( nZoomLevel > 0 && fabs( dfComputedRes - dfRes ) / dfRes > 1e-8 )
-        {
-            const char* pszZoomLevelStrategy = CSLFetchNameValueDef(papszOptions,
-                                                                    "ZOOM_LEVEL_STRATEGY",
-                                                                    "AUTO");
-            if( EQUAL(pszZoomLevelStrategy, "LOWER") )
-            {
-                nZoomLevel --;
-            }
-            else if( EQUAL(pszZoomLevelStrategy, "UPPER") )
-            {
-                /* do nothing */
-            }
-            else
-            {
-                if( dfPrevRes / dfComputedRes < dfComputedRes / dfRes )
-                    nZoomLevel --;
-            }
-            dfRes = tmList[nZoomLevel].mResX * tmList[0].mTileWidth / nBlockSize;
-        }
-
-        CPLDebug("COG", "Using ZOOM_LEVEL %d", nZoomLevel);
-
         const double dfTileExtent = dfRes * nBlockSize;
         int nTLTileX = static_cast<int>(std::floor((dfMinX - dfOriX) / dfTileExtent + 1e-10));
         int nTLTileY = static_cast<int>(std::floor((dfOriY - dfMaxY) / dfTileExtent + 1e-10));
@@ -941,10 +975,6 @@ GDALDataset* GDALCOGCreator::Create(const char * pszFilename,
                 pfnProgress, pProgressData );
         dfCurPixels = dfNextPixels;
 
-        // Used by GDALRegenerateOverviews() and GDALRegenerateOverviewsMultiBand()
-        CPLConfigOptionSetter oSetterRegeneratedBandIsMask(
-            "GDAL_REGENERATED_BAND_IS_MASK", "YES", true);
-
         CPLErr eErr = GTIFFBuildOverviewsEx(
             m_osTmpMskOverviewFilename,
             1, &poSrcMask,
@@ -1312,6 +1342,8 @@ void GDALCOGDriver::InitializeCreationOptionList()
 
     osOptions +=
 "  </Option>"
+"  <Option name='ZOOM_LEVEL' type='int' description='Target zoom level. "
+        "Only used for TILING_SCHEME != CUSTOM'/>"
 "  <Option name='ZOOM_LEVEL_STRATEGY' type='string-select' "
         "description='Strategy to determine zoom level. "
         "Only used for TILING_SCHEME != CUSTOM' default='AUTO'>"
@@ -1326,7 +1358,7 @@ void GDALCOGDriver::InitializeCreationOptionList()
 "  <Option name='EXTENT' type='string' description='"
         "Target extent as minx,miny,maxx,maxy for reprojection'/>"
 "  <Option name='ALIGNED_LEVELS' type='int' description='"
-        "Number of overview levels for which the tiles from GeoTIFF and the "
+        "Number of resolution levels for which the tiles from GeoTIFF and the "
         "specified tiling scheme match'/>"
 "  <Option name='ADD_ALPHA' type='boolean' description='Can be set to NO to "
         "disable the addition of an alpha band in case of reprojection' default='YES'/>"
@@ -1356,7 +1388,7 @@ void GDALRegister_COG()
     poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drivers/raster/cog.html" );
 
     poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES,
-                               "Byte UInt16 Int16 UInt32 Int32 Float32 "
+                               "Byte UInt16 Int16 UInt32 Int32 UInt64 Int64 Float32 "
                                "Float64 CInt16 CInt32 CFloat32 CFloat64" );
 
     poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
