@@ -35,7 +35,7 @@ extern "C" void GDALRegister_NULL();
 /*                          GDALNullDataset                             */
 /************************************************************************/
 
-class GDALNullDataset: public GDALDataset
+class GDALNullDataset final: public GDALDataset
 {
             int                 m_nLayers;
             OGRLayer**          m_papoLayers;
@@ -72,7 +72,7 @@ class GDALNullDataset: public GDALDataset
 /*                           GDALNullLayer                              */
 /************************************************************************/
 
-class GDALNullRasterBand: public GDALRasterBand
+class GDALNullRasterBand final: public GDALRasterBand
 {
     public:
             explicit GDALNullRasterBand(GDALDataType eDT);
@@ -92,7 +92,7 @@ class GDALNullRasterBand: public GDALRasterBand
 /*                           GDALNullLayer                              */
 /************************************************************************/
 
-class GDALNullLayer : public OGRLayer
+class GDALNullLayer final: public OGRLayer
 {
     OGRFeatureDefn      *poFeatureDefn;
     OGRSpatialReference *poSRS;
@@ -311,13 +311,13 @@ GDALDataset* GDALNullDataset::Open(GDALOpenInfo* poOpenInfo)
 /************************************************************************/
 
 GDALDataset *GDALNullDataset::Create(const char *,
-                                     int nXSize, int nYSize, int nBands,
+                                     int nXSize, int nYSize, int nBandsIn,
                                      GDALDataType eType, char **)
 {
     GDALNullDataset* poDS = new GDALNullDataset();
     poDS->nRasterXSize = nXSize;
     poDS->nRasterYSize = nYSize;
-    for( int i = 0; i < nBands; i++ )
+    for( int i = 0; i < nBandsIn; i++ )
         poDS->SetBand(i + 1, new GDALNullRasterBand(eType));
     return poDS;
 }
