@@ -55,7 +55,7 @@
 #include "ogr_spatialref.h"
 #include "ogr_srs_api.h"
 
-CPL_CVSID("$Id: envidataset.cpp 03855908e6edbe812777775bf3996e137d153a9d 2021-11-10 11:24:41 +0100 Even Rouault $")
+CPL_CVSID("$Id$")
 
 // TODO(schwehr): This really should be defined in port/somewhere.h.
 constexpr double kdfDegToRad = M_PI / 180.0;
@@ -507,7 +507,8 @@ void ENVIDataset::FlushCache(bool bAtClosing)
              poKey == "band names" ||
              poKey == "map info" ||
              poKey == "projection info" ||
-             poKey == "data ignore value" )
+             poKey == "data ignore value" ||
+             poKey == "coordinate system string" )
         {
             CSLDestroy(papszTokens);
             continue;
@@ -2682,7 +2683,7 @@ int ENVIDataset::GetEnviType(GDALDataType eType)
 /************************************************************************/
 
 GDALDataset *ENVIDataset::Create( const char *pszFilename,
-                                  int nXSize, int nYSize, int nBands,
+                                  int nXSize, int nYSize, int nBandsIn,
                                   GDALDataType eType,
                                   char **papszOptions )
 
@@ -2738,7 +2739,7 @@ GDALDataset *ENVIDataset::Create( const char *pszFilename,
 
     bool bRet = VSIFPrintfL(fp, "ENVI\n") > 0;
     bRet &= VSIFPrintfL(fp, "samples = %d\nlines   = %d\nbands   = %d\n",
-                        nXSize, nYSize, nBands) > 0;
+                        nXSize, nYSize, nBandsIn) > 0;
     bRet &=
         VSIFPrintfL(fp, "header offset = 0\nfile type = ENVI Standard\n") > 0;
     bRet &= VSIFPrintfL(fp, "data type = %d\n", iENVIType) > 0;
