@@ -73,7 +73,14 @@ CPL_C_END
 
 void OGR2SQLITE_Register()
 {
+#if !defined(SQLITE_HAS_NON_DEPRECATED_AUTO_EXTENSION) && defined(__GNUC__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated"
+#endif
     sqlite3_auto_extension ((void (*)(void)) OGR2SQLITE_static_register);
+#if !defined(SQLITE_HAS_NON_DEPRECATED_AUTO_EXTENSION) && defined(__GNUC__)
+#pragma GCC diagnostic pop
+#endif
 }
 
 #define VIRTUAL_OGR_DYNAMIC_EXTENSION_ENABLED
@@ -639,7 +646,22 @@ int OGR2SQLITE_ConnectCreate(sqlite3* hDB, void *pAux,
         osSQL += "\"";
         osSQL += " ";
         osSQL += OGRSQLiteFieldDefnToSQliteFieldDefn(poFieldDefn,
-                                                     bInternalUse);
+                                                     bInternalUse,
+                                                     false);
+        if( bInternalUse )
+        {
+            const auto& osDomainName = poFieldDefn->GetDomainName();
+            if( !osDomainName.empty() )
+            {
+                osSQL += "_BEGIN_DOMAIN_NAME_";
+                char* pszEncoded = CPLBinaryToHex(
+                    static_cast<int>(osDomainName.size()),
+                    reinterpret_cast<const GByte*>(osDomainName.data()));
+                osSQL += pszEncoded;
+                CPLFree(pszEncoded);
+                osSQL += "_END_DOMAIN_NAME";
+            }
+        }
     }
 
     if( bAddComma )
@@ -2586,6 +2608,23 @@ int sqlite3_extension_init (sqlite3 * hDB, char **pzErrMsg,
 
     *pzErrMsg = nullptr;
 
+    /* Check if we have been already loaded. */
+    /* This is to avoid 'ogrinfo :memory: --config OGR_SQLITE_LOAD_EXTENSIONS libgdal.so' to crash */
+    /* since it would run OGR2SQLITEModule::Setup() first with OGR2SQLITE_static_register() */
+    /* and then through here. */
+    int rc = sqlite3_exec(hDB, "SELECT ogr_version()", nullptr, nullptr, nullptr);
+
+    /* Reset error flag */
+    sqlite3_exec(hDB, "SELECT 1", nullptr, nullptr, nullptr);
+
+    if( rc == SQLITE_OK )
+    {
+
+        CPLDebug("OGR", "... OGR virtual OGR already loaded !");
+        *pzErrMsg = sqlite3_mprintf("Cannot load libgdal as an extension from a OGR SQLite datasource");
+        return SQLITE_ERROR;
+    }
+
     OGRRegisterAll();
 
     // Super hacky: this forces the malloc subsystem to be initialized.
