@@ -683,7 +683,6 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
         /* build rings */
         poLinearRingList.clear();
         OGRLinearRing *poOgrRing = nullptr;
-        int i = 1;
         for( PointListArray::const_iterator iRing = poRingList.begin(),
                  eRing = poRingList.end();
              iRing != eRing;
@@ -703,7 +702,6 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
                 OGRPoint *poPoint = &(*iPoint);
                 poOgrRing->addPoint(poPoint);
             }
-            i++;
         }
 
         /* find exterior ring */
