@@ -45,7 +45,7 @@
 #include "ogr_geometry.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: swq_op_general.cpp f74e78169b319b9465e0260da8e61c59362634ca 2021-06-14 19:24:54 +0200 Alessandro Pasotti $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                           swq_test_like()                            */
@@ -281,6 +281,10 @@ char* OGRHStoreGetValue(const char* pszHStore, const char* pszSearchedKey)
 /*                         OGRFormatDate()                              */
 /************************************************************************/
 
+#ifdef __GNUC__
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-function"
+#endif
 static const char * OGRFormatDate(const OGRField *psField)
 {
     return CPLSPrintf("%04d/%02d/%02d %02d:%02d:%06.3f",
@@ -291,6 +295,11 @@ static const char * OGRFormatDate(const OGRField *psField)
                     psField->Date.Minute,
                     psField->Date.Second );
 }
+
+#ifdef __GNUC__
+#pragma GCC diagnostic pop
+#endif
+
 #endif
 
 /************************************************************************/
