@@ -36,7 +36,7 @@
 
 #include <mutex>
 
-CPL_CVSID("$Id: cpl_odbc.cpp 279a37525abc9b4ccc44ec3c40f95772817ad3ca 2021-08-27 19:58:04 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 #ifndef SQLColumns_TABLE_CAT
 #define SQLColumns_TABLE_CAT 1
@@ -753,8 +753,14 @@ const char *CPLODBCSession::GetLastError()
 /*                          CPLODBCStatement()                          */
 /************************************************************************/
 
-/** Constructor */
-CPLODBCStatement::CPLODBCStatement( CPLODBCSession *poSession ) :
+/**
+ * Constructor.
+ *
+ * The optional flags argument can be used to specify flags which control
+ * the behavior of the statement.
+*/
+CPLODBCStatement::CPLODBCStatement( CPLODBCSession *poSession, const int flags ) :
+    m_nFlags(flags),
     m_poSession(poSession)
 {
 
@@ -857,6 +863,15 @@ int CPLODBCStatement::CollectResultsInfo()
         static_cast<char **>(CPLCalloc(sizeof(char *), m_nColCount + 1));
     m_panColValueLengths = static_cast<CPL_SQLLEN *>(
         CPLCalloc(sizeof(CPL_SQLLEN), m_nColCount + 1));
+    if ( m_nFlags & Flag::RetrieveNumericColumnsAsDouble )
+    {
+        m_padColValuesAsDouble =
+            static_cast<double *>(CPLCalloc(sizeof(double), m_nColCount + 1));
+    }
+    else
+    {
+        m_padColValuesAsDouble = nullptr;
+    }
 
     m_panColType =
         static_cast<SQLSMALLINT *>(CPLCalloc(sizeof(SQLSMALLINT), m_nColCount));
@@ -1143,30 +1158,21 @@ int CPLODBCStatement::Fetch( int nOrientation, int nOffset )
     if( nOrientation == SQL_FETCH_NEXT && nOffset == 0 )
     {
         nRetCode = SQLFetch( m_hStmt );
-        if( Failed(nRetCode) )
-        {
-            if( nRetCode != SQL_NO_DATA )
-            {
-                CPLError( CE_Failure, CPLE_AppDefined, "%s",
-                          m_poSession->GetLastError() );
-            }
-            return FALSE;
-        }
     }
     else
     {
         nRetCode = SQLFetchScroll(m_hStmt,
                                   static_cast<SQLSMALLINT>(nOrientation),
                                   nOffset);
-        if( Failed(nRetCode) )
+    }
+    if( Failed(nRetCode) )
+    {
+        if( nRetCode != SQL_NO_DATA )
         {
-            if( nRetCode == SQL_NO_DATA )
-            {
-                CPLError( CE_Failure, CPLE_AppDefined, "%s",
-                          m_poSession->GetLastError() );
-            }
-            return FALSE;
+            CPLError( CE_Failure, CPLE_AppDefined, "%s",
+                      m_poSession->GetLastError() );
         }
+        return FALSE;
     }
 
 /* -------------------------------------------------------------------- */
@@ -1175,6 +1181,8 @@ int CPLODBCStatement::Fetch( int nOrientation, int nOffset )
     for( SQLSMALLINT iCol = 0; iCol < m_nColCount; iCol++ )
     {
         CPL_SQLLEN cbDataLen = 0;
+        if ( m_padColValuesAsDouble )
+            m_padColValuesAsDouble[iCol] = std::numeric_limits< double >::quiet_NaN();
         SQLSMALLINT nFetchType = GetTypeMapping( m_panColType[iCol] );
 
         // Handle values other than WCHAR and BINARY as CHAR.
@@ -1183,9 +1191,47 @@ int CPLODBCStatement::Fetch( int nOrientation, int nOffset )
 
         char szWrkData[513] = {};
 
+        // If RetrieveNumericColumnsAsDouble flag is set, then read numeric columns using numeric data types and populate native double column values array.
+        // This allows retrieval of the original numeric value as a double via GetColDataAsDouble without risk of loss of precision.
+        // Additionally, some ODBC drivers (e.g. the MS Access ODBC driver) require reading numeric values using numeric
+        // data types, otherwise incorrect values can result. See https://github.com/OSGeo/gdal/issues/3885
+        if ( m_padColValuesAsDouble &&
+                ( m_panColType[iCol] == SQL_DOUBLE || m_panColType[iCol] == SQL_FLOAT || m_panColType[iCol] == SQL_REAL) )
+        {
+            if (m_panColType[iCol] == SQL_DOUBLE )
+            {
+                double dfValue = 0;
+                nRetCode = SQLGetData( m_hStmt, iCol + 1, SQL_C_DOUBLE,
+                            &dfValue, sizeof(dfValue), &cbDataLen );
+                if( cbDataLen != SQL_NULL_DATA )
+                    m_padColValuesAsDouble[iCol] = dfValue;
+            }
+            else
+            {
+                // note -- cannot request a float column as SQL_C_DOUBLE when using mdbtools driver!
+                float fValue = 0;
+                nRetCode = SQLGetData( m_hStmt, iCol + 1, SQL_C_FLOAT,
+                            &fValue, sizeof(fValue), &cbDataLen );
+                if( cbDataLen != SQL_NULL_DATA )
+                    m_padColValuesAsDouble[iCol] = static_cast< double >( fValue );
+            }
+            if( nRetCode != SQL_NO_DATA && Failed( nRetCode ) )
+            {
+                CPLError( CE_Failure, CPLE_AppDefined, "%s",
+                          m_poSession->GetLastError() );
+                return FALSE;
+            }
+            else
+            {
+                m_papszColValues[iCol] = nullptr;
+                m_panColValueLengths[iCol] = 0;
+                continue;
+            }
+        }
+
         nRetCode = SQLGetData( m_hStmt, iCol + 1, nFetchType,
-                               szWrkData, sizeof(szWrkData) - 1,
-                               &cbDataLen );
+                           szWrkData, sizeof(szWrkData) - 1,
+                           &cbDataLen );
 
         // SQLGetData() is giving garbage values in the first 4 bytes of
         // cbDataLen in some architectures. Converting it to int discards the
@@ -1215,7 +1261,7 @@ int CPLODBCStatement::Fetch( int nOrientation, int nOffset )
         // Assume big result: should check for state=SQLSATE 01004.
         else if( nRetCode == SQL_SUCCESS_WITH_INFO )
         {
-            if( cbDataLen >= static_cast<CPL_SQLLEN>(sizeof(szWrkData)-1) )
+          if( cbDataLen >= static_cast<CPL_SQLLEN>(sizeof(szWrkData)-1) || cbDataLen == SQL_NO_TOTAL )
             {
                 cbDataLen = static_cast<CPL_SQLLEN>(sizeof(szWrkData)-1);
                 if( nFetchType == SQL_C_CHAR )
@@ -1405,6 +1451,72 @@ int CPLODBCStatement::GetColDataLength( int iCol )
         return 0;
 }
 
+
+/************************************************************************/
+/*                        GetColDataAsDouble()                          */
+/************************************************************************/
+
+/**
+ * Fetch column data as a double value.
+ *
+ * Fetches the data contents of the requested column for the currently loaded
+ * row as a double value.
+ *
+ * Returns NaN if a non-numeric column is requested or the actual column value
+ * is "NULL".
+ *
+ * @warning this method can only be used if the Flag::RetrieveNumericColumnsAsDouble
+ * flag was set for the CPLODBCStatement.
+ *
+ * @param iCol the zero based column to fetch.
+ *
+ * @return numeric column value or NaN on failure.
+ */
+
+double CPLODBCStatement::GetColDataAsDouble( int iCol ) const
+
+{
+    if( !m_padColValuesAsDouble || iCol < 0 || iCol >= m_nColCount )
+        return std::numeric_limits< double >::quiet_NaN();
+    else
+        return m_padColValuesAsDouble[iCol];
+}
+
+/************************************************************************/
+/*                         GetColDataAsDouble()                         */
+/************************************************************************/
+
+/**
+ * Fetch column data as a double value.
+ *
+ * Fetches the data contents of the requested column for the currently loaded
+ * row as a double value.
+ *
+ * Returns NaN if a non-numeric column is requested or the actual column value
+ * is "NULL".
+ *
+ * @warning this method can only be used if the Flag::RetrieveNumericColumnsAsDouble
+ * flag was set for the CPLODBCStatement.
+ *
+ * @param pszColName the name of the column requested.
+ *
+ * @return numeric column value or NaN on failure.
+ */
+
+double CPLODBCStatement::GetColDataAsDouble( const char *pszColName ) const
+
+{
+    if( !m_padColValuesAsDouble )
+        return std::numeric_limits< double >::quiet_NaN();
+
+    const int iCol = GetColId( pszColName );
+
+    if( iCol == -1 )
+        return std::numeric_limits< double >::quiet_NaN();
+    else
+        return GetColDataAsDouble( iCol );
+}
+
 /************************************************************************/
 /*                              GetColId()                              */
 /************************************************************************/
@@ -1420,7 +1532,7 @@ int CPLODBCStatement::GetColDataLength( int iCol )
  * @return the column index, or -1 if not found.
  */
 
-int CPLODBCStatement::GetColId( const char *pszColName )
+int CPLODBCStatement::GetColId( const char *pszColName ) const
 
 {
     for( SQLSMALLINT iCol = 0; iCol < m_nColCount; iCol++ )
@@ -1680,11 +1792,17 @@ void CPLODBCStatement::Clear()
         CSLDestroy( m_papszColNames );
         m_papszColNames = nullptr;
 
-        CPLFree( m_papszColValues );
-        m_papszColValues = nullptr;
+        if ( m_papszColValues )
+        {
+            CPLFree( m_papszColValues );
+            m_papszColValues = nullptr;
+        }
 
         CPLFree( m_panColValueLengths );
         m_panColValueLengths = nullptr;
+
+        CPLFree( m_padColValuesAsDouble );
+        m_padColValuesAsDouble = nullptr;
     }
 }
 
